//       .
//  ,   .

//#include "stdafx.h"
#include <stdio.h>
#include "device.h"

//////////////////////////////////////////////////////////////////////////
//	Uncomment to use IOControl()
//	Sample uses alternate lookup method to find IOControl() in B181.dll
//#pragma comment (lib,"..\\_lib\\b181.lib")


#define PressEnter()\
{\
	printf("\nPress <ENTER>...");\
	UCHAR ch = getchar();\
}

#define OnExit()\
{\
	dev.CloseDevice();\
	PressEnter();\
}

//////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
//////////////////////////////////////////////////////////////////////////
{
	printf("\n==========================================================");
	printf("\nUNIPRO Analog Port B181 sample program (C)AURIS, 2003-2005");
	printf("\n==========================================================\n");
	
	// Finding device
	printf("\nTrying to find and initialize device......... ");
	CB181Device dev;
	int nErr = dev.PrepareDevice();
	switch (nErr) {
	case -1: printf("Error: B181.dll loading failed\n"); break;
	case -2: printf("Error: B181 IOControl() function is not found\n"); break;
	case -3: printf("Error: B181 device is not found\n"); break; //dev.DevicePresent == FALSE;
	default: break;
	};
	if (nErr) { OnExit(); return -1; };
	char str[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	dev.GetIDString(str);
	printf("Board ID: %s", str);

	printf("\nConnect DACCh1 to ADCCh1 if you wish to verify DAC output");
	PressEnter();
	
	// Configuring device
	printf("\nConfiguring device........................... ");
	dev.ConfigCmn();
	dev.ConfigADC();
	dev.ConfigDAC();
	printf("done.\n");
	
	// Writing DAC and reading ADC directly
	printf("\nWriting DACCh1 and reading ADCCh1 directly... ");
	for (int V = 9; V >= -9; V--)
	{
		printf("\nSetting DAC Ch1 = %+d V...  ", V);
		dev.WriteToDACDirect(float(V));
		printf("Reading ADC Ch1... ");
		float Vin = dev.ReadFromADCDirect();
		printf("%5.2f V.", Vin);
	}
	PressEnter();
	
	printf("\nWriting 1Vrms sine into DAC Ch1 memory....... ");
	float DacV[DAC_SAMPLES_PER_CH];
	for (int i = 0; i < DAC_SAMPLES_PER_CH; i++)
	{
		DacV[i] = float(1.4142 * sin(2*3.14159*i/DAC_SAMPLES_PER_CH));
//////////////////////////////////////////////////////////////////////////
//	Uncomment for square wave
//		if (DacV[i] < 0) DacV[i] = -2.f; else DacV[i] = +2.f;
//////////////////////////////////////////////////////////////////////////
	}
	
	dev.WriteToDAC(DacV);
	printf("done\n");

	printf("\nStarting DAC and ADC simultaneously.......... ");
	dev.RunD(TRUE, TRUE);
	printf("done\n");

	printf("\nWaiting for ADC acquisition complete......... ");
	// Readind device status
	CmnStatusStruct status;
	int nTimes = 0;
	do
	{
		dev.ReadStatus(status);
		nTimes++;
		Sleep(1);
	}
	while (!status.m_bADC_DataReady && (nTimes < 3000));
	printf(nTimes ? "done (%d)\n":"timeout (%d)\n", nTimes);

	printf("\nReading ADCData.............................. ");
	float AdcV[ADC_SAMPLES_PER_CH];
	dev.ReadFromADC(AdcV);
	printf("done");

	printf("\nADC acquisition data");
	for (i = 0; i < 10; i++)
	{
		printf("\n");
		for (int j = 0; j < 10; j++)
			printf("%6.2f", AdcV[10*i+j]);
	}

	PressEnter();
	
	printf("\nStopping generation.......................... ");
	dev.StopD();
//	dev.ResetD();
	printf("done\n");
	
	OnExit();
	return 0;
}

