
#include <stdio.h>
#include "device.h"

//-------------     -----------------------------------
int CB181Device::PrepareDevice()
{
	DevicePresent = FALSE;

	dllInstall = LoadLibrary("b181.dll");
	if (!dllInstall) return -1;

	IOCONTROL = (pIOControl*)GetProcAddress(dllInstall,"IOControl");
	if ( IOCONTROL == NULL ) return -2;
	
	try { IOCONTROL(IOCTRL_Initialize, DeviceID, NULL); }
	catch (...) { return -3;}

	DevicePresent = TRUE;
	if (!IOCONTROL(IOCTRL_InitClbrInfo, DeviceID, &m_ClbrInfo)) return -4;
	
	return 0;
};

void CB181Device::GetIDString(char* str)
{
	ULONG nVer = DeviceID.m_nDeviceVersion;
	ULONG nSer = nVer & 0xFFFF;
	sprintf(str, "B%d%d%d.%d\n", nVer>>24, DeviceID.m_DeviceType, (nVer>>16) & 0xFF, nSer);
}

//-------------     -----------------------------------
void CB181Device::CloseDevice() {
	if (DevicePresent)
	{
		IOCONTROL(IOCTRL_Deinitialize, DeviceID, 0);
		DevicePresent = FALSE;
		IOCONTROL = NULL;
	};
	if (DllPresent) FreeLibrary(dllInstall);
};

 //---------------------   ---------------------------------------
void CB181Device::ConfigCmn() {
	CmnConfigStruct cc;
	cc.m_cExtMem0Source = MemSrc_ADC0;
	cc.m_cExtMem1Source = MemSrc_DAC0;
	cc.m_cIntMem0Source = MemSrc_LA0;
	cc.m_cIntMem1Source = MemSrc_LG0;
	cc.m_bLD3_0Direction = TRUE;
	cc.m_bLD7_4Direction = TRUE;
	cc.m_bLD11_8Direction = TRUE;
	cc.m_bLD15_12Direction = TRUE;
	cc.m_wInternalScale0 = 500;
	cc.m_wInternalScale1 = 5000;
	IOCONTROL(IOCTRL_CmnConfig, DeviceID, &cc);
};

//---------------------   ---------------------------------------
void CB181Device::ConfigADC() {
	ADCConfigStruct ca;
	ca.m_bRecorder = FALSE;
	ca.m_cChannels = 0x1; // ADC channels mask (4 LSB - 4 channels) // 0x5 - Ch1 and Ch3
	ca.m_cRunSource = RunSrc_SW;
	ca.m_cClockSource = ClockSrc_IntGen0;
	ca.m_wWorkLen = ADC_SAMPLES_PER_CH;
	IOCONTROL(IOCTRL_ADCConfig, DeviceID, &ca);
};

//---------------------   ---------------------------------------
void CB181Device::ConfigDAC() {
	DACConfigStruct cd;
	cd.m_bSingleCycle = FALSE;
	cd.m_bEndlessGenerator = FALSE;
	cd.m_cChannels = 0; // DAC number (0-Ch1, 1-Ch2, 2-Ch3, 3-Ch4)
	cd.m_cRunSource = RunSrc_SW;
	cd.m_cClockSource = ClockSrc_IntGen0;
	cd.m_wWorkLen = DAC_SAMPLES_PER_CH;
	IOCONTROL(IOCTRL_DACConfig, DeviceID, &cd);
};

//---------------------   ---------------------------------
BOOL CB181Device::ReadStatus(CmnStatusStruct& status) {
//	cs.m_bADC_Collision;
//	cs.m_bDAC_Collision;
//	cs.m_bLA_Collision;
//	cs.m_bLG_Collision;
//	cs.m_bADC_DataReady;
//	cs.m_bADC_Mem1Full;
//	cs.m_bADC_Mem0Full;
//	cs.m_bDAC_Mem1Empty;
//	cs.m_bDAC_Mem0Empty;
//	cs.m_bLA_Mem1Full;
//	cs.m_bLA_Mem0Full;
//	cs.m_bLG_Mem1Empty;
//	cs.m_bLG_Mem0Empty;
	IOCONTROL(IOCTRL_CmnReadStatus, DeviceID, &status);
	return TRUE;
};

WORD CB181Device::VoltageToDACCode(BYTE ch, float fValue)
{
	double Coef = (m_ClbrInfo.m_DACCodePlus10[ch] - m_ClbrInfo.m_DACCodeMinus10[ch]) / 20.;
	return max(0, min( 4095, (short)floor(0.5 + (fValue + 10) * Coef + m_ClbrInfo.m_DACCodeMinus10[ch]) ) );
}

float CB181Device::ADCCodeToVoltage(BYTE ch, WORD nCode)
{
//	ADC data are in 14-bit two's complement code.
//	return float((( (short(nCode<<2))>>2) - m_ClbrInfo.m_ADCZeroCode[ch]) * m_ClbrInfo.m_ADCCoef[ch]);
	return float(((nCode ^ 0x2000) - 8192 - m_ClbrInfo.m_ADCZeroCode[ch]) * m_ClbrInfo.m_ADCCoef[ch]);
}


//-----------------    ----------------------------------------
//   16- (  -  WORD)
void CB181Device::ReadFromADC(float * SignalIn) {
	CmnControlStruct cc;
	cc.m_bADC_Run = cc.m_bADC_StartSingleRead = FALSE;
	cc.m_bDAC_Reset = cc.m_bDAC_SW_Stop = cc.m_bDAC_Run = cc.m_bDAC_MemCntClear = FALSE;
	cc.m_bLA_Reset  = cc.m_bLA_SW_Stop  = cc.m_bLA_Run  = FALSE;
	cc.m_bLG_Reset  = cc.m_bLG_SW_Stop  = cc.m_bLG_Run  = cc.m_bLG_MemCntClear = FALSE;
	cc.m_bADC_Reset = cc.m_bADC_SW_Stop = TRUE;
	IOCONTROL(IOCTRL_CmnControl, DeviceID, &cc);

	ReadADCStruct ra;
	ra.m_pData = (WORD*)MemExt0;
	ra.m_cDataSource = DtSrc_Mem0;
	ra.m_wCount = ADC_SAMPLES_PER_CH;
	IOCONTROL(IOCTRL_ADCReadData, DeviceID, &ra);
	for(int i = 0; i < ADC_SAMPLES_PER_CH; i++)
		SignalIn[i] = ADCCodeToVoltage( 0, ra.m_pData[i] );
};

float CB181Device::ReadFromADCDirect() { // this function returns only Ch1 code
	CmnControlStruct cc;
	cc.m_bADC_Reset = cc.m_bADC_SW_Stop = cc.m_bADC_Run = FALSE;
	cc.m_bDAC_Reset = cc.m_bDAC_SW_Stop = cc.m_bDAC_Run = cc.m_bDAC_MemCntClear = FALSE;
	cc.m_bLA_Reset  = cc.m_bLA_SW_Stop  = cc.m_bLA_Run  = FALSE;
	cc.m_bLG_Reset  = cc.m_bLG_SW_Stop  = cc.m_bLG_Run  = cc.m_bLG_MemCntClear = FALSE;
	cc.m_bADC_StartSingleRead = TRUE;
	IOCONTROL(IOCTRL_CmnControl, DeviceID, &cc);
	Sleep(1);

	// Always 4 samples. Number of valid samples was defined by ADC channels mask (ca.m_cChannels)
	WORD Data[4] = {0,0,0,0};

	ReadADCStruct ra;
	ra.m_pData = Data;
	ra.m_cDataSource = DtSrc_DirectChannels;
	IOCONTROL(IOCTRL_ADCReadData, DeviceID, &ra);
	return ADCCodeToVoltage( 0, Data[0] );
};

//------------------    ------------------------------------------
//   32- (  -  DWORD; HIWORD  )
void CB181Device::WriteToDAC(float *SignalOut) {
	CmnControlStruct cc;
	cc.m_bADC_Reset = cc.m_bADC_SW_Stop = cc.m_bADC_Run = cc.m_bADC_StartSingleRead = FALSE;
	cc.m_bDAC_SW_Stop = cc.m_bDAC_Run = FALSE;
	cc.m_bLA_Reset = cc.m_bLA_SW_Stop = cc.m_bLA_Run = FALSE;
	cc.m_bLG_Reset = cc.m_bLG_SW_Stop = cc.m_bLG_Run = cc.m_bLG_MemCntClear = FALSE;
	cc.m_bDAC_Reset = cc.m_bDAC_MemCntClear = TRUE;
	IOCONTROL(IOCTRL_CmnControl, DeviceID, &cc);

	WriteDACCStruct wd;
	wd.m_cDataDestination = DtSrc_Mem0;
	wd.m_pData = MemInt0;
	wd.m_wCount = DAC_SAMPLES_PER_CH;
	wd.m_cChannel = 0; // 0-Ch1, 1-Ch2, 2-Ch3, 3-Ch4
	wd.m_bLDAC_ = FALSE;
	for(int i = 0; i < DAC_SAMPLES_PER_CH; i++)
		wd.m_pData[i] = VoltageToDACCode(0, SignalOut[i]);
	IOCONTROL(IOCTRL_DACWriteData, DeviceID, &wd);
}

void CB181Device::WriteToDACDirect(float fValue) {
	ULONG dwCode = VoltageToDACCode(0, fValue);
	WriteDACCStruct wd;
	wd.m_cDataDestination = DtSrc_DirectChannels;
	wd.m_pData = &dwCode;
	wd.m_cChannel = 0; // 0-Ch1, 1-Ch2, 2-Ch3, 3-Ch4
	wd.m_bLDAC_ = FALSE; // FALSE to load
	IOCONTROL(IOCTRL_DACWriteData, DeviceID, &wd);
}

//---------------------   -----------------------------------
void CB181Device::ResetD() {
	CmnControlStruct cc;
	cc.m_bADC_StartSingleRead = FALSE;
	cc.m_bADC_SW_Stop = cc.m_bADC_Run = FALSE;
	cc.m_bDAC_SW_Stop = cc.m_bDAC_Run = FALSE;
	cc.m_bLA_SW_Stop = cc.m_bLA_Run = FALSE;
	cc.m_bLG_SW_Stop = cc.m_bLG_Run = FALSE;

	cc.m_bADC_Reset = TRUE;

	cc.m_bDAC_Reset = TRUE;
	cc.m_bDAC_MemCntClear = TRUE;

	cc.m_bLA_Reset = TRUE;
	cc.m_bLG_Reset = TRUE;
	cc.m_bLG_MemCntClear = TRUE;

	IOCONTROL(IOCTRL_CmnControl, DeviceID, &cc);
}

//------------------------   --------------------------------
void CB181Device::RunD(BOOL bADC, BOOL bDAC) {
	CmnControlStruct cc;
	cc.m_bADC_SW_Stop = cc.m_bADC_Run = cc.m_bADC_StartSingleRead = FALSE;
	cc.m_bDAC_SW_Stop = cc.m_bDAC_Run = FALSE;
	cc.m_bLA_Reset = cc.m_bLA_SW_Stop = cc.m_bLA_Run = FALSE;
	cc.m_bLG_Reset = cc.m_bLG_SW_Stop = cc.m_bLG_Run = cc.m_bLG_MemCntClear = FALSE;
	cc.m_bADC_Reset = bADC;
	cc.m_bDAC_Reset = cc.m_bDAC_MemCntClear = bDAC;
	IOCONTROL(IOCTRL_CmnControl, DeviceID, &cc);

	cc.m_bADC_Reset = cc.m_bADC_SW_Stop = cc.m_bADC_StartSingleRead = FALSE;
	cc.m_bDAC_Reset = cc.m_bDAC_SW_Stop = cc.m_bDAC_MemCntClear = FALSE;
	cc.m_bLA_Reset = cc.m_bLA_SW_Stop = cc.m_bLA_Run = FALSE;
	cc.m_bLG_Reset = cc.m_bLG_SW_Stop = cc.m_bLG_Run = cc.m_bLG_MemCntClear = FALSE;
	cc.m_bADC_Run = bADC;
	cc.m_bDAC_Run = bDAC;
	IOCONTROL(IOCTRL_CmnControl, DeviceID, &cc);
}

//------------------------   ------------------------------
void CB181Device::StopD() {
	CmnControlStruct cc;
	cc.m_bADC_Reset = cc.m_bADC_Run = cc.m_bADC_StartSingleRead = FALSE;
	cc.m_bDAC_Reset = cc.m_bDAC_Run = cc.m_bDAC_MemCntClear = FALSE;
	cc.m_bLA_Reset = cc.m_bLA_Run = cc.m_bLA_SW_Stop = FALSE;
	cc.m_bLG_Reset = cc.m_bLG_Run = cc.m_bLG_SW_Stop = cc.m_bLG_MemCntClear = FALSE;
	
	cc.m_bADC_SW_Stop = TRUE;
	cc.m_bDAC_SW_Stop = TRUE;
	IOCONTROL(IOCTRL_CmnControl, DeviceID, &cc);
}
